@@ -5,7 +5,7 @@
 #' @description This function is useful for inheriting argument documentation
 #'   in functions that create custom third-party launchers. See
 #'   `@inheritParams crew::crew_launcher` in the source code file of
-#'   [crew_launcher_callr()].
+#'   [crew_launcher_local()].
 #' @param name Name of the launcher.
 #' @param seconds_launch Seconds of startup time to allow.
 #'   A worker is unconditionally assumed to be alive
@@ -41,20 +41,25 @@
 #' @param tasks_timers Number of tasks to do before activating
 #'   the timers for `seconds_idle` and `seconds_wall`.
 #'   See the `timerlaunch` argument of `mirai::server()`.
-#' @param async_dial Logical, whether the `mirai` workers should dial in
-#'   asynchronously. See the `asyncdial` argument of `mirai::server()`.
-#' @param cleanup Logical, whether to clean up global options and the
-#'   global environment after every task.
-#'   See the `cleanup` argument of `mirai::server()`.
+#' @param reset_globals `TRUE` to reset global environment
+#'   variables between tasks, `FALSE` to leave them alone.
+#' @param reset_packages `TRUE` to unload any packages loaded during
+#'   a task (runs between each task), `FALSE` to leave packages alone.
+#' @param reset_options `TRUE` to reset global options to their original
+#'   state between each task, `FALSE` otherwise. It is recommended to
+#'   only set `reset_options = TRUE` if `reset_packages` is also `TRUE`
+#'   because packages sometimes rely on options they set at loading time.
+#' @param garbage_collection `TRUE` to run garbage collection between
+#'   tasks, `FALSE` to skip.
 #' @examples
 #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
 #' crew_session_start()
 #' router <- crew_router()
-#' router$listen()
-#' launcher <- crew_launcher_callr()
+#' router$start()
+#' launcher <- crew_launcher_local()
 #' launcher$populate(sockets = router$sockets)
 #' launcher$launch()
-#' m <- mirai::mirai("result")
+#' m <- mirai::mirai("result", .compute = router$name)
 #' Sys.sleep(0.25)
 #' m$data
 #' router$terminate()
@@ -63,18 +68,20 @@
 crew_launcher <- function(
   name = NULL,
   seconds_launch = 30,
-  seconds_interval = 0.001,
+  seconds_interval = 0.01,
   seconds_timeout = 10,
   seconds_idle = Inf,
   seconds_wall = Inf,
   seconds_exit = 0.1,
   tasks_max = Inf,
   tasks_timers = 0L,
-  async_dial = TRUE,
-  cleanup = FALSE
+  reset_globals = TRUE,
+  reset_packages = FALSE,
+  reset_options = FALSE,
+  garbage_collection = FALSE
 ) {
-  name <- as.character(name %|||% random_name())
-  launcher <- crew_class_launcher_callr$new(
+  name <- as.character(name %|||% crew_random_name())
+  launcher <- crew_class_launcher_local$new(
     name = name,
     seconds_launch = seconds_launch,
     seconds_interval = seconds_interval,
@@ -84,8 +91,10 @@ crew_launcher <- function(
     seconds_exit = seconds_exit,
     tasks_max = tasks_max,
     tasks_timers = tasks_timers,
-    async_dial = async_dial,
-    cleanup = cleanup
+    reset_globals = reset_globals,
+    reset_packages = reset_packages,
+    reset_options = reset_options,
+    garbage_collection = garbage_collection
   )
   launcher$validate()
   launcher
@@ -100,11 +109,11 @@ crew_launcher <- function(
 #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
 #' crew_session_start()
 #' router <- crew_router()
-#' router$listen()
-#' launcher <- crew_launcher_callr()
+#' router$start()
+#' launcher <- crew_launcher_local()
 #' launcher$populate(sockets = router$sockets)
 #' launcher$launch()
-#' m <- mirai::mirai("result")
+#' m <- mirai::mirai("result", .compute = router$name)
 #' Sys.sleep(0.25)
 #' m$data
 #' router$terminate()
@@ -116,14 +125,7 @@ crew_class_launcher <- R6::R6Class(
   portable = TRUE,
   public = list(
     #' @field workers Data frame of worker information.
-    workers = tibble::tibble(
-      socket = character(0L),
-      launches = integer(0L),
-      start = numeric(0L),
-      token = character(0L),
-      listener = list(),
-      handle = list()
-    ),
+    workers = NULL,
     #' @field name Name of the launcher.
     name = NULL,
     #' @field seconds_launch See [crew_launcher()].
@@ -142,10 +144,14 @@ crew_class_launcher <- R6::R6Class(
     tasks_max = NULL,
     #' @field tasks_timers See [crew_launcher()].
     tasks_timers = NULL,
-    #' @field async_dial See [crew_launcher()].
-    async_dial = NULL,
-    #' @field cleanup See [crew_launcher()].
-    cleanup = NULL,
+    #' @field reset_globals See [crew_launcher()].
+    reset_globals = NULL,
+    #' @field reset_packages See [crew_launcher()].
+    reset_packages = NULL,
+    #' @field reset_options See [crew_launcher()].
+    reset_options = NULL,
+    #' @field garbage_collection See [crew_launcher()].
+    garbage_collection = NULL,
     #' @description Launcher constructor.
     #' @return An `R6` object with the launcher.
     #' @param name See [crew_launcher()].
@@ -153,22 +159,23 @@ crew_class_launcher <- R6::R6Class(
     #' @param seconds_interval See [crew_launcher()].
     #' @param seconds_timeout See [crew_launcher()].
     #' @param seconds_idle See [crew_launcher()].
-    #'   See the `idletime` argument of `mirai::server()`.
     #' @param seconds_wall See [crew_launcher()].
     #' @param seconds_exit See [crew_launcher()].
     #' @param tasks_max See [crew_launcher()].
     #' @param tasks_timers See [crew_launcher()].
-    #' @param async_dial See [crew_launcher()].
-    #' @param cleanup See [crew_launcher()].
+    #' @param reset_globals See [crew_launcher()].
+    #' @param reset_packages See [crew_launcher()].
+    #' @param reset_options See [crew_launcher()].
+    #' @param garbage_collection See [crew_launcher()].
     #' @examples
     #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
     #' crew_session_start()
     #' router <- crew_router()
-    #' router$listen()
-    #' launcher <- crew_launcher_callr()
+    #' router$start()
+    #' launcher <- crew_launcher_local()
     #' launcher$populate(sockets = router$sockets)
     #' launcher$launch()
-    #' m <- mirai::mirai("result")
+    #' m <- mirai::mirai("result", .compute = router$name)
     #' Sys.sleep(0.25)
     #' m$data
     #' router$terminate()
@@ -184,8 +191,10 @@ crew_class_launcher <- R6::R6Class(
       seconds_exit = NULL,
       tasks_max = NULL,
       tasks_timers = NULL,
-      async_dial = NULL,
-      cleanup = NULL
+      reset_globals = NULL,
+      reset_packages = NULL,
+      reset_options = NULL,
+      garbage_collection = NULL
     ) {
       self$name <- name
       self$seconds_launch <- seconds_launch
@@ -196,13 +205,43 @@ crew_class_launcher <- R6::R6Class(
       self$seconds_exit <- seconds_exit
       self$tasks_max <- tasks_max
       self$tasks_timers <- tasks_timers
-      self$async_dial <- async_dial
-      self$cleanup <- cleanup
+      self$reset_globals <- reset_globals
+      self$reset_packages <- reset_packages
+      self$reset_options <- reset_options
+      self$garbage_collection <- garbage_collection
     },
     #' @description Validate the launcher.
     #' @return `NULL` (invisibly).
     validate = function() {
-      true(self$name, is.character(.), length(.) == 1L, !anyNA(.), nzchar(.))
+      crew_assert(
+        is.function(self$launch_worker),
+        message = "launch_worker() must be a function."
+      )
+      names <- c("call", "launcher", "worker", "instance")
+      crew_assert(
+        names %in% names(formals(self$launch_worker)),
+        message = paste(
+          "launch_worker() must have arguments \"call\", \"launcher\",",
+          "\"worker\", and \"instance\"."
+        )
+      )
+      if (!is.null(self$terminate_worker)) {
+        crew_assert(
+          is.function(self$terminate_worker),
+          message = "terminate_worker() must be a function or NULL."
+        )
+        crew_assert(
+          "handle" %in% names(formals(self$terminate_worker)),
+          message = "terminate_worker() must have a \"handle\" argument."
+        )
+      }
+      crew_assert(
+        self$name,
+        is.character(.),
+        length(.) == 1L,
+        !anyNA(.),
+        nzchar(.)
+      )
       fields <- c(
         "seconds_launch",
         "seconds_interval",
@@ -214,13 +253,29 @@ crew_class_launcher <- R6::R6Class(
         "tasks_timers"
       )
       for (field in fields) {
-        true(self[[field]], is.numeric(.), . >= 0, length(.) == 1L, !anyNA(.))
+        crew_assert(
+          self[[field]],
+          is.numeric(.),
+          . >= 0,
+          length(.) == 1L,
+          !anyNA(.)
+        )
       }
-      for (field in c("async_dial", "cleanup")) {
-        true(self[[field]], isTRUE(.) || isFALSE(.))
+      fields <- c(
+        "reset_globals",
+        "reset_packages",
+        "reset_options",
+        "garbage_collection"
+      )
+      for (field in fields) {
+        crew_assert(self[[field]], isTRUE(.) || isFALSE(.))
+      }
+      if (!is.null(self$workers)) {
+        crew_assert(self$workers, is.data.frame(.))
+        cols <- c("handle", "socket", "start", "launches")
+        crew_assert(identical(colnames(self$workers), cols))
+        crew_assert(nrow(self$workers) > 0L)
       }
-      true(self$async_dial, isTRUE(.) || isFALSE(.))
-      true(self$workers, is.data.frame(.))
       invisible()
     },
     #' @description List of arguments for `mirai::server()`.
@@ -228,160 +283,135 @@ crew_class_launcher <- R6::R6Class(
     #' @param socket Character of length 1, websocket address of the worker
     #'   to launch.
     settings = function(socket) {
+      cleanup <- as.integer(isTRUE(self$reset_globals)) +
+        (2L * as.integer(isTRUE(self$reset_packages))) +
+        (4L * as.integer(isTRUE(self$reset_options))) +
+        (8L * as.integer(isTRUE(self$garbage_collection)))
       list(
         url = socket,
-        asyncdial = self$async_dial,
         maxtasks = self$tasks_max,
         idletime = self$seconds_idle * 1000,
         walltime = self$seconds_wall * 1000,
         timerstart = self$tasks_timers,
         exitlinger = self$seconds_exit * 1000,
-        cleanup = self$cleanup
+        cleanup = cleanup,
+        asyncdial = FALSE
       )
     },
     #' @description Create a call to [crew_worker()] to
     #'   help create custom launchers.
     #' @return Character of length 1 with a call to [crew_worker()].
     #' @param socket Socket where the worker will receive tasks.
-    #' @param host IP address of the `mirai` client that sends tasks.
-    #' @param port TCP port to register a successful connection
-    #'   to the host. Different from that of `socket`.
-    #' @param token Character of length 1 to identify the instance
-    #'   of the `mirai` server process connected to the socket.
-    #' @param name User-supplied name of the launcher, useful for
-    #'   constructing informative job labels.
+    #' @param launcher Character of length 1, name of the launcher.
+    #' @param worker Positive integer of length 1, index of the worker.
+    #'   This worker index remains the same even when the current instance
+    #'   of the worker exits and a new instance launches.
+    #' @param instance Character of length 1 to uniquely identify
+    #'   the instance of the worker.
     #' @examples
-    #' launcher <- crew_launcher_callr()
+    #' launcher <- crew_launcher_local()
     #' launcher$call(
-    #'   socket = "ws://127.0.0.1:5000",
-    #'   host = "127.0.0.1",
-    #'   port = "5711",
-    #'   token = "my_token",
-    #'   name = "my_name"
+    #'   socket = "ws://127.0.0.1:5000/3/cba033e58",
+    #'   launcher = "launcher_a",
+    #'   worker = 3L,
+    #'   instance = "cba033e58"
     #' )
-    call = function(socket, host, port, token, name) {
-      args <- self$settings(socket)
-      args$.fn <- "list"
-      call_settings <- do.call(what = rlang::call2, args = args)
+    call = function(socket, launcher, worker, instance) {
       call <- substitute(
         crew::crew_worker(
-          token = token,
-          host = host,
-          port = port,
-          settings = settings
+          settings = settings,
+          launcher = launcher,
+          worker = worker,
+          instance = instance
         ),
         env = list(
-          settings = call_settings,
-          host = host,
-          port = port,
-          token = token
+          settings = self$settings(socket),
+          launcher = launcher,
+          worker = worker,
+          instance = instance
         )
       )
       out <- deparse_safe(expr = call, collapse = " ")
       gsub(pattern = "   *", replacement = " ", x = out)
     },
-    #' @description Populate the workers data frame.
+    #' @description Start the launcher.
+    #' @details Creates the workers data frame.
+    #'   Meant to be called once at the beginning of the launcher
+    #'   life cycle.
     #' @return `NULL` (invisibly).
-    #' @param sockets Character vector of worker websockets.
-    populate = function(sockets) {
+    #' @param workers Positive integer of length 1,
+    #'   number of workers to allow.
+    start = function(workers = 1L) {
+      crew_assert(workers > 0L, message = "workers must be > 0")
       self$workers <- tibble::tibble(
-        socket = as.character(sockets),
-        launches = rep(0L, length(sockets)),
-        start = rep(NA_real_, length(sockets)),
-        token = rep(NA_character_, length(sockets)),
-        listener = replicate(length(sockets), crew_null, simplify = FALSE),
-        handle = replicate(length(sockets), crew_null, simplify = FALSE)
+        handle = replicate(workers, crew_null, simplify = FALSE),
+        socket = rep(NA_character_, workers),
+        start = rep(NA_real_, workers),
+        launches = rep(0L, workers)
       )
       invisible()
     },
-    #' @description Get the active workers.
-    #' @details An active worker is a worker that should be given the chance
-    #'   to run tasks. To determine if the worker is active,
-    #'   `crew` monitors seconds past launch time, and it listens
-    #'   to a special non-`mirai` NNG websocket that the worker
-    #'   is supposed to dial into on launch.
-    #'   If the worker is currently connected to the websocket,
-    #'   then it is active. Otherwise, if the worker is not connected
-    #'   and the startup window has expired, the worker is inactive.
-    #'   Otherwise, if the worker is not connected and the startup
-    #'   window has not yet expired, then the worker is active
-    #'   if it has not ever connected to a websocket.
-    #' @return Character vector of worker websockets.
-    active = function() {
-      listeners <- self$workers$listener
+    #' @description Launch a worker.
+    #' @return `NULL` (invisibly).
+    #' @param index Positive integer of length 1, index of the worker
+    #'   to launch.
+    #' @param socket Character of length 1, sockets of the worker to launch.
+    launch = function(index, socket = NULL) {
+      if (!length(socket) || !is.character(socket)) {
+        return(invisible())
+      }
+      instance <- parse_instance(socket)
+      call <- self$call(
+        socket = socket,
+        launcher = self$name,
+        worker = index,
+        instance = instance
+      )
+      handle <- self$launch_worker(
+        call = call,
+        launcher = self$name,
+        worker = index,
+        instance = instance
+      )
+      self$workers$handle[[index]] <- handle
+      self$workers$socket[index] <- socket
+      self$workers$start[index] <- nanonext::mclock() / 1000
+      self$workers$launches[[index]] <- self$workers$launches[[index]] + 1L
+      invisible()
+    },
+    #' @description Show which workers are expected to launch.
+    #' @return A logical vector indicating which workers are expected
+    #'   to launch.
+    expected = function() {
+      !is.na(self$workers$socket)
+    },
+    #' @description Show whether each worker is launching.
+    #' @details A worker is considered "launching" if it was started
+    #'   recently (`seconds_launch` seconds ago or sooner).
+    #' @return A logical vector indicating which workers are launching.
+    launching = function() {
       bound <- self$seconds_launch
       start <- self$workers$start
-      now <- bench::hires_time()
-      listening <- map_lgl(listeners, connection_opened)
-      connected <- map_lgl(listeners, dialer_connected)
-      launching <- !is.na(start) & ((now - start) < bound)
-      discovered <- connected
-      index <- listening & (!connected) & launching
-      discovered[index] <- map_lgl(listeners[index], dialer_discovered)
-      active <- listening & (connected | (launching & (!discovered)))
-      self$workers$socket[active]
-    },
-    #' @description Launch one or more workers.
-    #' @details If a worker is already assigned to a socket,
-    #'   the previous worker is terminated before the next
-    #'   one is launched.
-    #' @return `NULL` (invisibly).
-    #' @param sockets Sockets where the workers will dial in.
-    launch = function(sockets = character(0)) {
-      true(
-        !is.null(crew_session_port()),
-        message = "call crew_session_start() before launching workers."
-      )
-      matches <- match(x = sockets, table = self$workers$socket)
-      true(!anyNA(matches), message = "bad websocket on launch.")
-      for (index in matches) {
-        handle <- self$workers$handle[[index]]
-        if (!is_crew_null(handle)) {
-          self$terminate_worker(handle)
-        }
-        socket <- self$workers$socket[index]
-        self$workers$start[index] <- bench::hires_time()
-        token <- random_name()
-        self$workers$token[index] <- token
-        listener <- connection_listen(
-          host = crew_session_host(),
-          port = crew_session_port(),
-          token = token
-        )
-        handle <- self$launch_worker(
-          socket = socket,
-          host = crew_session_host(),
-          port = crew_session_port(),
-          token = token,
-          name = self$name
-        )
-        self$workers$listener[[index]] <- listener
-        self$workers$handle[[index]] <- handle
-        self$workers$launches[[index]] <- self$workers$launches[[index]] + 1L
-      }
-      invisible()
+      now <- nanonext::mclock() / 1000
+      !is.na(start) & ((now - start) < bound)
     },
     #' @description Terminate one or more workers.
     #' @return `NULL` (invisibly).
-    #' @param sockets Character vector of sockets of the workers
-    #'   to terminate.
-    terminate = function(sockets = NULL) {
-      sockets <- sockets %|||% self$workers$socket
-      matches <- match(x = sockets, table = self$workers$socket)
-      true(!anyNA(matches), message = "bad websocket on terminate.")
-      for (index in matches) {
+    #' @param indexes Integer vector of the indexes of the workers
+    #'   to terminate. If `NULL`, all current workers are terminated.
+    terminate = function(indexes = NULL) {
+      if (is.null(self$workers)) {
+        return(invisible())
+      }
+      indexes <- indexes %|||% seq_len(nrow(self$workers))
+      for (index in indexes) {
         handle <- self$workers$handle[[index]]
         if (!is_crew_null(handle)) {
           self$terminate_worker(handle)
         }
-        con <- self$workers$listener[[index]]
-        if (!is_crew_null(con) && connection_opened(con)) {
-          close(con)
-        }
-        self$workers$start[index] <- NA_real_
-        self$workers$token[index] <- NA_character_
-        self$workers$listener[[index]] <- crew_null
         self$workers$handle[[index]] <- crew_null
+        self$workers$socket[index] <- NA_character_
+        self$workers$start[index] <- NA_real_
       }
       invisible()
     },
@@ -390,8 +420,9 @@ crew_class_launcher <- R6::R6Class(
     #'   placeholder, and its presence allows manual worker termination
     #'   to be optional.
     #' @return `NULL` (invisibly).
-    #' @param handle A `callr` process handle previously
-    #'   returned by `launch_worker()`.
+    #' @param handle A handle object previously
+    #'   returned by `launch_worker()` which allows the termination
+    #'   of the worker.
     terminate_worker = function(handle) {
       invisible()
     }
